Package: ETDQualitizer
Type: Package
Title: Automated Eye Tracking Data Quality Determination for
        Screen-Based Eye Trackers
Version: 1.1.0
Authors@R: c(
    person(
      "Diederick", "Niehorster",
      email = "diederick_c.niehorster@humlab.lu.se",
      role = c("aut", "cre", "cph"),
      comment = c(ORCID = "0000-0002-4672-8756")
    )
  )
Maintainer: Diederick Niehorster <diederick_c.niehorster@humlab.lu.se>
Description: Compute common data quality metrics for accuracy, precision and data loss
  for screen-based eye trackers. The package supports gaze input in screen pixels or
  degrees and reports angular measures in degrees where appropriate. If you use this
  package, please cite Niehorster, D.C., Nyström, M., Hessels, R.S., Benjamins, J.S.,
  Andersson, R., and Hooge, I.T.C. (2026). The fundamentals of eye tracking, Part 7:
  Determining data quality. Behavior Research Methods. <doi:10.3758/s13428-026-03039-4>.
License: MIT + file LICENSE
URL: https://github.com/dcnieho/ETDQualitizer
BugReports: https://github.com/dcnieho/ETDQualitizer/issues
Encoding: UTF-8
Imports: R6, stats
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-06-07 22:17:16 UTC; runner
Author: Diederick Niehorster [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-4672-8756>)
Repository: CRAN
Date/Publication: 2026-06-08 06:50:19 UTC
Built: R 4.5.2; ; 2026-06-08 10:01:26 UTC; unix
