# Generated by roxygen2: do not edit by hand

export(DataQuality)
export(ETDQ_version)
export(Fick_to_vector)
export(ScreenConfiguration)
export(accuracy)
export(bcea)
export(compute_data_quality_from_validation)
export(data_loss_from_expected)
export(data_loss_from_invalid)
export(effective_frequency)
export(precision_using_moving_window)
export(report_data_quality_table)
export(rms_s2s)
export(std)
export(vector_to_Fick)
importFrom(R6,R6Class)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,median)
importFrom(stats,sd)
